statemachine class CKNGEredinIceSpike extends W3UsableItem
{
	editable var explodeAfter : float;
	editable var damageRadius : float;
	editable var damageVal : float;
	editable var effectDuration : float;
	
	editable var timeBetweenSpawn : float;
	editable var minDistFromTarget : float;
	editable var maxDistFromTarget : float;
	editable var minDistFromEachOther : float;
	
	var meshComp : CMeshComponent;
	var destructionComp	: CDestructionSystemComponent;
	var entitiesInRange : array< CGameplayEntity >;
	var possibleUsedEffectTypes : array<EEffectType>;	
	
	var frostNovaEntityTemplate : CEntityTemplate;
	var frostNovaEntity : CKNGEredinIceSpike;	
	
	var target : CActor;
	var targetPosition : Vector;
	var targetRotation : EulerAngles;
	
	var surfaceFX : SFXSurfacePostParams;
	var postEffect : CGameplayFXSurfacePost;
	
	default explodeAfter = 2.0;
	default damageRadius = 2.5;
	default damageVal = 50;
	default effectDuration = 4.0;
	
	default timeBetweenSpawn = 0.5;
	default minDistFromTarget = 1.5;
	default maxDistFromTarget = 3.0;
	default minDistFromEachOther = 1.5; 
	
	event OnUsed (usedBy : CEntity) {
	
		
		Init();
	
	}
	
	function Init() {
			
		target = thePlayer.GetTarget();
		postEffect = theGame.GetSurfacePostFX();
		
		if(target && thePlayer.IsInCombat()) {
			
			GotoState('WhiteFrostNova');
			
		}	
	}
	
	event OnSpawned( spawnData : SEntitySpawnData )
	{
		destructionComp = (CDestructionSystemComponent)GetComponentByClassName( 'CDestructionSystemComponent' );
		meshComp = (CMeshComponent)GetComponentByClassName( 'CMeshComponent' );
		
		frostNovaEntityTemplate = (CEntityTemplate)LoadResource("dlc\dlckngwhitefrostnova\data\w2ent\kng_eredin_ice_spike.w2ent",true);
		
		if( meshComp )
		{
			meshComp.SetVisible( false );
		}
		
		GotoStateAuto();
		
	}

	public function Appear()
	
	{
		meshComp.SetVisible( true );
		PlayEffect( 'appear' );
		PlayEffect( 'tell' );
		AddTimer( 'Explode', explodeAfter );
		DestroyAfter( explodeAfter + 5.0 );
	}
	
	timer function Explode( deltaTime : float, optional id : int )
	{
		var damage : W3DamageAction;
		var i : int;
		var actor : CActor;
		
		StopAllEffects();
		PlayEffect( 'explosion' );
		meshComp.SetVisible( false );
		
		GCameraShake( 0.5, true, GetWorldPosition(), 20.0f );

		FindGameplayEntitiesInRange( entitiesInRange, this, damageRadius, 1000 );	
		
		for( i = 0; i < entitiesInRange.Size(); i += 1 )
		{
			actor = (CActor)entitiesInRange[i];
			if( actor )
			{
			
				possibleUsedEffectTypes.Clear();
				possibleUsedEffectTypes.PushBack(EET_Stagger);
				possibleUsedEffectTypes.PushBack(EET_LongStagger);
				possibleUsedEffectTypes.PushBack(EET_Paralyzed);				
			
				damage = new W3DamageAction in this;
				damage.Initialize( thePlayer, entitiesInRange[i], NULL, this, EHRT_Heavy, CPS_Undefined, false, false, false, true );
				damage.AddDamage( theGame.params.DAMAGE_NAME_FROST, damageVal );
				//damage.AddEffectInfo( EET_Snowstorm, effectDuration );
				damage.AddEffectInfo( possibleUsedEffectTypes[i], effectDuration );
				theGame.damageMgr.ProcessAction( damage );
				
				delete damage;				
			}
		}		
	}	
}

state Idle in CKNGEredinIceSpike {}

state WhiteFrostNova in CKNGEredinIceSpike {

	var usedPos : array<Vector>;
	
	event OnEnterState( prevStateName : name )
	{	
		Run();
	}
	
	event OnLeaveState( nextStateName : name )
	{
	
	}
	
	entry function Run()
	{
		var pos : Vector;
		var i : int;
		
		usedPos.Clear();
		
		while( VecDistance2D( parent.target.GetWorldPosition(), parent.GetWorldPosition() ) < 100.0 )
		{
			pos = FindPosition();
			
			while( !IsPositionValid( pos ) )
			{
				SleepOneFrame();
				pos = FindPosition();
			}
			
			Spawn( pos );
			
			usedPos.PushBack( pos );
			if( usedPos.Size() > 5 )
				usedPos.Clear();
			Sleep( parent.timeBetweenSpawn );
		}
	}
	
	function Spawn( position : Vector )	{
				
		var spawnPos : Vector;
		var rotation : EulerAngles;
		var collisionGroups : array<name>;
		
		
				
		if( parent.frostNovaEntityTemplate )
		{
			collisionGroups.PushBack( 'Terrain' );
			collisionGroups.PushBack( 'Static' );
		
			spawnPos = position;
								
			parent.frostNovaEntity = (CKNGEredinIceSpike)theGame.CreateEntity(parent.frostNovaEntityTemplate, spawnPos, rotation);
			
			if(parent.frostNovaEntity) {
			
				parent.frostNovaEntity.Appear();	
				parent.postEffect.AddSurfacePostFXGroup(spawnPos,2,10,2,15,0);   
			
			}											
		}
	}

		function FindPosition() : Vector {
		
		var randVec : Vector = Vector( 0.f, 0.f, 0.f );
		var targetPos : Vector;
		var outPos : Vector;
		
		targetPos = parent.target.GetWorldPosition();
		randVec = VecRingRand( parent.minDistFromTarget, parent.maxDistFromTarget );
		
		outPos = targetPos + randVec;
		
		return outPos;
	}
	
	protected function IsPositionValid( out whereTo : Vector ) : bool {
	
		var newPos : Vector;
		var radius : float;
		var z : float;
		var i : int;

		radius = parent.target.GetRadius();
		
		if( !theGame.GetWorld().NavigationFindSafeSpot( whereTo, radius, radius*3, newPos ) )
		{
			if( theGame.GetWorld().NavigationComputeZ( whereTo, whereTo.Z - 5.0, whereTo.Z + 5.0, z ) )
			{
				whereTo.Z = z;
				if( !theGame.GetWorld().NavigationFindSafeSpot( whereTo, radius, radius*3, newPos ) )
					return false;
			}
			return false;
		}
		
		for( i = 0; i < usedPos.Size(); i += 1 )
		{
			if( VecDistance2D( newPos, usedPos[i] ) < parent.minDistFromEachOther )
				return false;
		}
		

		whereTo = newPos;
		
		return true;
	}
	
	
	
	
	
	
	
	
	
	
	
	

}